import functools
from collections.abc import Callable, Coroutine, Iterable
from typing import Any, TypeVar

from .adaptive_async_concurrency_limiter import ServiceOverloadError

T = TypeVar("T")

OVERLOAD_KEYWORDS = (
    "overload",
    "temporarily unavailable",
    "service unavailable",
    "too many requests",
    "rate limit",
    "rate limited",
    "try again",
    "retry",
    "busy",
    "too many",
)


def raise_on_overload(
    overload_keywords: tuple[str, ...] = OVERLOAD_KEYWORDS,
    cared_exception: type[Exception]
    | Callable[[Exception], bool]
    | Iterable[Callable[[Exception], bool] | type[Exception]] = Exception,
) -> Callable[
    [Callable[..., Coroutine[Any, Any, T]]], Callable[..., Coroutine[Any, Any, T]]
]:
    """将包含过载关键词的 Exception 转换为 ServiceOverloadError。

    Args:
        overload_keywords: 要视为过载的关键词元组，默认为 OVERLOAD_KEYWORDS
        cared_exception: 需要捕获的异常类型或者一个输入为异常对象的函数

    Returns:
        装饰器函数，用于包装异步函数

    Raises:
        ServiceOverloadError: 当响应包含过载关键词时
    """
    if not isinstance(cared_exception, Iterable):
        cared_exception = (cared_exception,)

    def is_cared_exception(e: Exception) -> bool:
        for cared_e in cared_exception:
            if isinstance(cared_e, type):
                if isinstance(e, cared_e):
                    return True
            elif callable(cared_e) and cared_e(e) is True:
                return True
        return False

    def decorator(
        func: Callable[..., Coroutine[Any, Any, T]],
    ) -> Callable[..., Coroutine[Any, Any, T]]:
        @functools.wraps(func)
        async def wrapper(*args: Any, **kwargs: Any) -> T:
            try:
                return await func(*args, **kwargs)
            except Exception as e:
                if is_cared_exception(e):
                    exception_str = str(e)
                    if any(keyword in exception_str for keyword in overload_keywords):
                        raise ServiceOverloadError(e) from e
                raise e

        return wrapper

    return decorator



import asyncio
from collections.abc import Awaitable, Callable, Coroutine
from functools import wraps
from typing import Any, TypeVar

from adaptio.log_utils import setup_colored_logger

T = TypeVar("T")

# 设置logger
logger = setup_colored_logger(__name__)


class FakeLock:
    """空锁实现，用于不需要并发控制时"""

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        return None


def with_async_control(
    cared_exception: type[Exception]
    | tuple[type[Exception], ...]
    | Callable[[Exception], bool] = Exception,
    max_concurrency: int = 0,
    max_qps: float = 0,
    retry_n: int = 0,
    retry_delay: float = 1.0,
) -> Callable[[Callable[..., Awaitable[T]]], Callable[..., Coroutine[Any, Any, T]]]:
    """
    异步函数的装饰器，提供并发限制、QPS控制和重试功能

    参数:
        cared_exception: 需要捕获的异常类型或者一个输入为异常对象的函数
        max_concurrency: 最大并发数
        max_qps: 每秒最大请求数 (0表示不限制)
        retry_n: 重试次数
        retry_delay: 重试间隔时间(秒)

    返回:
        装饰器函数
    """
    retry_n = int(max(retry_n, 0))

    def decorator(
        func: Callable[..., Awaitable[T]],
    ) -> Callable[..., Coroutine[Any, Any, T]]:
        # 为每个装饰器实例创建独立的锁
        concurrency_sem = (
            asyncio.Semaphore(max_concurrency) if max_concurrency > 0 else FakeLock()
        )
        qps_lock = asyncio.Lock()

        @wraps(func)
        async def wrapper(*args, **kwargs) -> T:
            async with concurrency_sem:
                for attempt in range(retry_n + 1):
                    try:
                        if max_qps > 1e-5:  # 避免浮点数精度问题
                            async with qps_lock:
                                await asyncio.sleep(1 / max_qps)
                        return await func(*args, **kwargs)
                    except Exception as e:
                        if retry_n <= 0:
                            raise
                        if callable(cared_exception):
                            if not cared_exception(e):
                                raise
                        elif not isinstance(e, cared_exception):
                            raise
                        logger.error(
                            f"（{attempt + 1}/{retry_n}） 尝试 {func.__name__} 失败: \n Class: {e.__class__.__name__}\n Message: {e}"
                        )
                        if attempt >= retry_n:
                            logger.error(
                                f"（{attempt + 1}/{retry_n}） 尝试 {func.__name__} 达到最大次数！"
                            )
                            raise
                        await asyncio.sleep(retry_delay)
                raise Exception("所有重试都失败了")

        return wrapper

    return decorator


if __name__ == "__main__":
    import time

    @with_async_control(
        cared_exception=ValueError,
        max_concurrency=5,
        max_qps=10,
        retry_n=3,
        retry_delay=0.5,
    )
    async def test_api(i: int) -> str:
        # 模拟一个可能失败的API调用
        if i % 3 == 2:  # 让每三个请求中的一个失败
            raise ValueError(f"模拟 ValueError错误 - 请求 {i}")
        if i % 3 == 1:
            raise RuntimeError(f"模拟 RuntimeError 错误 - 请求 {i}")
        await asyncio.sleep(1.0)  # 模拟API延迟
        return f"请求 {i} 成功"

    async def main():
        print("开始测试...")
        start_time = time.time()

        # 创建5个并发任务
        tasks = [test_api(i) for i in range(10)]

        # 打印结果
        for i, future in enumerate(asyncio.as_completed(tasks)):
            try:
                result = await future
                logger.info(f"任务 {i} 成功: {result}")
            except Exception as e:
                logger.warning(
                    f"任务 {i} 失败: \n Class: {e.__class__.__name__}\n Message: {e}"
                )

        end_time = time.time()
        print(f"\n总耗时: {end_time - start_time:.2f}秒")

    # 运行测试
    asyncio.run(main())



import asyncio
from collections.abc import Coroutine

from .adjustable_semaphore import AdjustableSemaphore
from .log_utils import setup_colored_logger


class ServiceOverloadError(BaseException):
    pass


class AdaptiveAsyncConcurrencyLimiter:
    """自适应并发限制器，用于动态控制并发任务数量。

    使用类似 TCP 拥塞控制的算法来动态调整并发数：
    - 当系统正常运行时，逐步增加并发数
    - 当检测到过载时，快速降低并发数
    - 通过动态信号量（DynamicSemaphore）来控制并发度

    Args:
        max_concurrency: 最大允许的并发数
        min_concurrency: 最小允许的并发数
        initial_concurrency: 初始并发数
        adjust_overload_rate: 触发并发度调整的过载率阈值
            意思是在最近一轮并发调用中，若触发过载错误的调用数量超过这个比例，才会进行降低并发数操作
        overload_exception: 用于标识过载的异常类型
        log_level: 日志级别
        log_prefix: 日志前缀
        ignore_loop_bound_exception: 是否忽略事件循环绑定异常
            如果你获取了一个在另一个asyncio循环中初始化的信号量，实际上它没有任何限制并发的能力！
            默认情况下，这个库在这种情况下会引发RuntimeError异常。
            但是，如果你将此选项设置为True，它将忽略异常，并且除了打印一条 warning 外没有其他动作。
            通常情况下很难在实际应用中出发这个错误，除非刻意写出在同步函数中使用多线程调用异步函数的代码。
            https://github.com/python/cpython/blob/v3.13.3/Lib/asyncio/mixins.py#L20
    """

    def __init__(
        self,
        max_concurrency=256,
        min_concurrency=1,
        initial_concurrency=1,
        adjust_overload_rate: float = 0.1,
        overload_exception: type[BaseException] = ServiceOverloadError,
        log_level: str = "INFO",
        log_prefix: str = "",
        ignore_loop_bound_exception: bool = False,
    ):
        if initial_concurrency < min_concurrency:
            raise ValueError(
                f"{log_prefix} -- {initial_concurrency=} 不能小于 {min_concurrency=}"
            )
        if initial_concurrency > max_concurrency:
            raise ValueError(
                f"{log_prefix} -- {initial_concurrency=} 不能大于 {max_concurrency=}"
            )
        if min_concurrency > max_concurrency:
            raise ValueError(
                f"{log_prefix} -- {min_concurrency=} 不能大于 {max_concurrency=}"
            )

        self.max_concurrency = max_concurrency
        self.min_concurrency = min_concurrency
        self.adjust_overload_rate = adjust_overload_rate
        self.overload_exception = overload_exception
        self.log_prefix = log_prefix

        self.logger = setup_colored_logger(
            logger_name=f"scheduler_{id(self)}",
            log_level=log_level,
        )

        self.submitted_tasks: set[asyncio.Future] = set()

        self.current_failed_count = 0
        self.current_overload_count = 0
        self.current_succeed_count = 0
        self.current_finished_count = 0
        self.current_running_count = 0

        self.workers_lock = AdjustableSemaphore(
            initial_concurrency,
            ignore_loop_bound_exception=ignore_loop_bound_exception,
        )
        # 添加新的变量来跟踪调整状态
        self.increase_step = 1  # 初始增长步长
        self.decrease_factor = 0.75  # 遇到过载时的下降因子

    def reset_counters(self):
        self.current_failed_count = 0
        self.current_overload_count = 0
        self.current_succeed_count = 0
        self.current_finished_count = 0

    async def adjust_concurrency(self):
        """借鉴TCP的拥塞控制算法调整 workers 数量"""
        if self.current_finished_count == 0:
            self.logger.debug(f"{self.log_prefix} -- 没有完成的任务，跳过调整")
            return

        overload_rate = self.current_overload_count / self.current_finished_count
        self.logger.debug(
            f"{self.log_prefix} -- 当前过载率: {overload_rate:.2%}, 调整阈值: {self.adjust_overload_rate:.2%}"
        )

        if overload_rate > self.adjust_overload_rate:
            # 遇到过载时，乘以下降因子快速降低并发数
            new_concurrency = max(
                self.min_concurrency,
                int(self.workers_lock.initial_value * self.decrease_factor),
            )
            self.increase_step = 1  # 重置增长步长
            self.logger.info(
                f"{self.log_prefix} -- 检测到过载，降低并发数至 {new_concurrency}"
            )
        else:
            # 未过载时，采用渐进式增长
            new_concurrency = min(
                self.max_concurrency,
                self.workers_lock.initial_value + self.increase_step,
            )
            self.increase_step = min(
                self.increase_step * 2, 16
            )  # 指数增长步长，但设置上限
            self.logger.info(
                f"{self.log_prefix} -- 系统运行正常，提升并发数从 {self.workers_lock.initial_value} 到 {new_concurrency}，下次增长步长: {self.increase_step}"
            )

        await self.workers_lock.set_value(new_concurrency)

    def submit(self, coro: Coroutine):
        if not self.workers_lock.initial_value:
            raise RuntimeError("并发限制器已关闭")

        async def _task_wrapper():
            async with self.workers_lock:
                self.current_running_count += 1
                try:
                    result = await coro
                    self.current_succeed_count += 1
                    return result
                except self.overload_exception:
                    self.current_overload_count += 1
                    self.logger.debug(
                        f"{self.log_prefix} -- "
                        f"服务过载，当前触发过载任务数: {self.current_overload_count} "
                        f"任务状态 - 已完成: {self.current_finished_count}, "
                        f"成功数: {self.current_succeed_count}, "
                        f"运行中: {self.current_running_count}, "
                        f"过载数: {self.current_overload_count}, "
                        f"失败数: {self.current_failed_count}, "
                        f"当前并发度: {self.workers_lock.get_value()}, "
                        f"基准并发度: {self.workers_lock.initial_value}",
                    )
                    raise
                except Exception:
                    self.current_failed_count += 1
                    raise
                finally:
                    self.current_finished_count += 1
                    self.current_running_count -= 1
                    self.logger.debug(
                        f"{self.log_prefix} -- "
                        f"任务状态 - 已完成: {self.current_finished_count}, "
                        f"成功数: {self.current_succeed_count}, "
                        f"运行中: {self.current_running_count}, "
                        f"过载数: {self.current_overload_count}, "
                        f"失败数: {self.current_failed_count}, "
                        f"当前并发度: {self.workers_lock.get_value()}, "
                        f"基准并发度: {self.workers_lock.initial_value}"
                    )
                    if self.workers_lock.get_value() < 0:
                        self.reset_counters()

                    if self.current_finished_count > self.workers_lock.initial_value:
                        await self.adjust_concurrency()
                        self.reset_counters()

        def _on_done(task):
            # self.finished_tasks.put_nowait(task)
            self.submitted_tasks.remove(task)

        task = asyncio.create_task(_task_wrapper())
        task.add_done_callback(_on_done)
        self.submitted_tasks.add(task)
        return task

    async def shutdown(self):
        """关闭并发限制器，等待所有任务完成"""
        await self.workers_lock.set_value(0)
        if self.submitted_tasks:
            await asyncio.gather(*self.submitted_tasks, return_exceptions=True)
        self.submitted_tasks.clear()



import logging

import colorlog


def setup_colored_logger(
    logger_name: str,
    log_level: str = "INFO",
    propagate: bool = False,
) -> logging.Logger:
    """配置一个带有彩色输出的日志记录器

    Args:
        logger_name: 日志记录器名称
        log_level: 日志级别，默认为 "INFO"
        propagate: 是否传播日志到父级logger，默认为False

    Returns:
        配置好的 Logger 实例
    """
    logger = logging.getLogger(logger_name)
    # 清除已存在的处理器
    logger.handlers.clear()
    # 设置日志传播
    logger.propagate = propagate

    handler = colorlog.StreamHandler()
    formatter = colorlog.ColoredFormatter(
        "%(log_color)s%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        log_colors={
            "DEBUG": "cyan",
            "INFO": "green",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITICAL": "red,bg_white",
        },
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(getattr(logging, log_level.upper()))

    return logger



import asyncio
import logging
from collections.abc import Callable, Coroutine
from functools import wraps
from typing import Any, TypeVar

from adaptio.adaptive_async_concurrency_limiter import (
    AdaptiveAsyncConcurrencyLimiter,
    ServiceOverloadError,
)

R = TypeVar("R")


def with_adaptive_retry(
    scheduler: AdaptiveAsyncConcurrencyLimiter | None = None,
    max_retries: int = 1024,
    retry_interval_seconds: float = 1,
    max_concurrency: int = 256,
    min_concurrency: int = 1,
    initial_concurrency: int = 1,
    adjust_overload_rate: float = 0.1,
    overload_exception: type[BaseException] = ServiceOverloadError,
    log_level: str = "INFO",
    log_prefix: str = "",
    ignore_loop_bound_exception: bool = False,
) -> Callable[
    [Callable[..., Coroutine[Any, Any, R]]], Callable[..., Coroutine[Any, Any, R]]
]:
    """装饰器：为异步函数添加自适应重试机制。

    当函数触发过载异常时，会自动重试并通过 AdaptiveConcurrencyLimiter 动态调整并发数。

    Args:
        scheduler: AdaptiveConcurrencyLimiter 实例。如果为 None，则为每个装饰的函数创建独立的限制器
        max_retries: 最大重试次数
        retry_interval_seconds: 重试间隔时间（秒）
        max_concurrency: 当 scheduler 为 None 时使用的最大并发数
        min_concurrency: 当 scheduler 为 None 时使用的最小并发数
        initial_concurrency: 当 scheduler 为 None 时使用的初始并发数
        adjust_overload_rate: 当 scheduler 为 None 时使用的过载调整率
            意思是在最近一轮并发调用中，若触发过载错误的调用数量超过这个比例，才会进行降低并发数操作
        overload_exception: 当 scheduler 为 None 时检测的过载异常类型
        log_level: 当 scheduler 为 None 时使用的日志级别
        log_prefix: 当 scheduler 为 None 时使用的日志前缀
        ignore_loop_bound_exception: 是否忽略循环边界异常
            如果你获取了一个在另一个asyncio循环中初始化的信号量，实际上它没有任何限制并发的能力！
            默认情况下，这个库在这种情况下会引发RuntimeError异常。
            但是，如果你将此选项设置为True，它将忽略异常，并且除了打印一条 warning 外没有其他动作。
            通常情况下很难在实际应用中出发这个错误，除非刻意写出在同步函数中使用多线程调用异步函数的代码。
            https://github.com/python/cpython/blob/v3.13.3/Lib/asyncio/mixins.py#L20

    Returns:
        装饰后的异步函数，具有自适应重试能力
    """
    # 如果没有传入 scheduler，则创建一个新的限制器实例
    _scheduler = scheduler or AdaptiveAsyncConcurrencyLimiter(
        max_concurrency=max_concurrency,
        min_concurrency=min_concurrency,
        initial_concurrency=initial_concurrency,
        adjust_overload_rate=adjust_overload_rate,
        overload_exception=overload_exception,
        log_level=log_level,
        log_prefix=log_prefix,
        ignore_loop_bound_exception=ignore_loop_bound_exception,
    )

    def decorator(
        func: Callable[..., Coroutine[Any, Any, R]],
    ) -> Callable[..., Coroutine[Any, Any, R]]:
        if not _scheduler.log_prefix:
            _scheduler.log_prefix = getattr(func, "__name__", "unnamed_function")

        @wraps(func)
        async def wrapper(*args: Any, **kwargs: Any) -> R:
            retries = 0
            # 为装饰器创建独立的 logger
            retry_logger = logging.getLogger(f"retry_{id(func)}")
            while True:
                try:
                    task = _scheduler.submit(func(*args, **kwargs))
                    return await task  # type: ignore
                except _scheduler.overload_exception:
                    retries += 1
                    if retries > max_retries:
                        retry_logger.error(
                            f"{_scheduler.log_prefix} -- 重试次数已达上限({retries}次)，服务仍处于过载状态"
                        )
                        raise
                    await asyncio.sleep(retry_interval_seconds)
                    continue

        return wrapper

    return decorator


if __name__ == "__main__":
    import random

    # 设计一个达到 32 并发就会触发 ServiceOverloadError 的测试任务
    sample_task_overload_threshold = 32
    sample_task_running_count = 0

    async def sample_task(task_id):
        """A sample task that simulates workload and triggers overload at a certain concurrency."""
        global sample_task_running_count
        sample_task_running_count += 1
        # 模拟随机任务耗时
        await asyncio.sleep(random.uniform(1, 3))
        # 模拟过载错误

        if sample_task_running_count > sample_task_overload_threshold:
            sample_task_running_count -= 1
            logging.error(
                f"===sample_task {sample_task_running_count} tasks > {sample_task_overload_threshold}==="
            )
            raise ServiceOverloadError(
                f"Service overloaded with {sample_task_running_count} tasks > {sample_task_overload_threshold}"
            )
        else:
            sample_task_running_count -= 1
        return f"Task {task_id} done"

    @with_adaptive_retry(initial_concurrency=4, log_level="INFO")
    async def sample_task_with_retry(task_id):
        return await sample_task(task_id)

    async def get_result():
        tasks = [sample_task_with_retry(i) for i in range(1000)]
        for res in asyncio.as_completed(tasks):
            try:
                logging.info(f"SUCCESS: {await res}")
            except Exception as e:
                logging.error(f"error: {e}")
                pass

    asyncio.run(get_result())



import asyncio

from loguru import logger


class AdjustableSemaphore:
    """可调整容量的异步信号量

    这个信号量允许在运行时动态调整最大并发数。

    Args:
        initial_value (int): 初始的信号量值（最大并发数）

    Raises:
        ValueError: 当尝试设置负数值时抛出
    """

    def __init__(
        self, initial_value: int = 1, ignore_loop_bound_exception: bool = False
    ) -> None:
        """
        Args:
            ignore_loop_bound_exception: whether to ignore the loop bound exception
            https://github.com/python/cpython/blob/v3.13.3/Lib/asyncio/mixins.py#L20
            If you acquire a semaphore which inited in a different asyncio loop, it actually doesn't have any ability to limit the concurrency!
            As default, this library will raise a RuntimeError in this case.
            But if you set this option to True, it will ignore the exception and do NOTHING!
        """
        if initial_value < 0:
            raise ValueError("Initial semaphore value cannot be negative")
        self.initial_value = initial_value
        self._current_value = initial_value
        self._condition = asyncio.Condition()
        self.ignore_loop_bound_exception = ignore_loop_bound_exception

    async def acquire(self) -> bool:
        """获取信号量"""
        async with self._condition:
            while self._current_value <= 0:
                await self._condition.wait()
            self._current_value -= 1
            return True

    async def release(self) -> None:
        """释放信号量"""
        async with self._condition:
            self._current_value += 1
            self._condition.notify(1)

    async def set_value(self, value: int) -> None:
        """动态设置新的并发数量"""
        if value < 0:
            raise ValueError("Semaphore value cannot be negative")

        async with self._condition:
            delta = value - self.initial_value
            self.initial_value = value
            self._current_value += delta

            # 如果新值增加了，唤醒等待的协程
            if delta > 0:
                self._condition.notify(delta)

    def get_value(self) -> int:
        """获取当前信号量的值"""
        return self._current_value

    async def __aenter__(self):
        try:
            await self.acquire()
        except RuntimeError as e:
            if (
                "is bound to a different event loop" in str(e)
                and self.ignore_loop_bound_exception
            ):
                logger.warning(
                    f"Catched the loop bound exception: {e}, but ignored it because ignore_loop_bound_exception is True, this semaphore is actually not working!"
                )
            else:
                raise e

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        try:
            await self.release()
        except RuntimeError as e:
            if (
                "is bound to a different event loop" in str(e)
                and self.ignore_loop_bound_exception
            ):
                logger.warning(
                    f"Catched the loop bound exception: {e}, but ignored it because ignore_loop_bound_exception is True, this semaphore is actually not working!"
                )
            else:
                raise e


if __name__ == "__main__":
    import random
    import time

    current_working_count = 0

    async def worker(sem: AdjustableSemaphore, worker_id: int):
        """模拟工作任务"""
        async with sem:
            global current_working_count
            current_working_count += 1
            print(
                f"[{time.strftime('%H:%M:%S')}] 工作者 {worker_id} 开始执行，当前并发数：{sem.initial_value - sem.get_value()} 当前工作协程数：{current_working_count}"
            )
            await asyncio.sleep(random.uniform(2, 5))  # 模拟耗时操作
            print(f"[{time.strftime('%H:%M:%S')}] 工作者 {worker_id} 完成")
            current_working_count -= 1

    async def dynamic_controller(sem: AdjustableSemaphore):
        """动态控制并发数量"""
        print(f"\n[{time.strftime('%H:%M:%S')}] 初始并发数为 {sem.initial_value}")
        await asyncio.sleep(10)  # 等待一些任务开始执行

        print(f"\n[{time.strftime('%H:%M:%S')}] 将并发数调整为 2")
        await sem.set_value(2)
        await asyncio.sleep(10)

        print(f"\n[{time.strftime('%H:%M:%S')}] 将并发数调整为 7")
        await sem.set_value(7)
        await asyncio.sleep(10)

        print(f"\n[{time.strftime('%H:%M:%S')}] 将并发数调整为 3")
        await sem.set_value(3)
        await asyncio.sleep(10)

        print(f"\n[{time.strftime('%H:%M:%S')}] 将并发数调整为 1")
        await sem.set_value(1)
        await asyncio.sleep(10)

        print(f"\n[{time.strftime('%H:%M:%S')}] 将并发数调整为 0")
        await sem.set_value(0)
        await asyncio.sleep(10)

        print(f"\n[{time.strftime('%H:%M:%S')}] 将并发数调整为 20")
        await sem.set_value(20)

    async def main():
        # 初始并发数为 3
        sem = AdjustableSemaphore(5)

        # 创建 100 个工作任务
        workers = [worker(sem, i) for i in range(100)]

        # 创建动态控制任务
        controller = dynamic_controller(sem)

        # 同时运行所有任务
        await asyncio.gather(controller, *workers)

    if __name__ == "__main__":
        asyncio.run(main())



import functools
from collections.abc import Callable, Coroutine
from typing import Any, TypeVar

import aiohttp

from .adaptive_async_concurrency_limiter import ServiceOverloadError

OVERLOAD_STATUS_CODES = (503, 429)

T = TypeVar("T")


def raise_on_aiohttp_overload(
    overload_status_codes: tuple[int, ...] = OVERLOAD_STATUS_CODES,
) -> Callable[
    [Callable[..., Coroutine[Any, Any, T]]], Callable[..., Coroutine[Any, Any, T]]
]:
    """将 aiohttp 的特定状态码错误转换为 ServiceOverloadError。

    Args:
        overload_status_codes: 要视为过载的 HTTP 状态码元组，默认为 (503, 429)

    Returns:
        装饰器函数，用于包装异步函数

    Raises:
        ServiceOverloadError: 当响应状态码在 overload_status_codes 中时
        aiohttp.ClientResponseError: 其他 HTTP 错误
    """

    def decorator(
        func: Callable[..., Coroutine[Any, Any, T]],
    ) -> Callable[..., Coroutine[Any, Any, T]]:
        @functools.wraps(func)
        async def wrapper(*args: Any, **kwargs: Any) -> T:
            try:
                return await func(*args, **kwargs)
            except aiohttp.ClientResponseError as e:
                if e.status in overload_status_codes:
                    raise ServiceOverloadError(e) from e
                raise e

        return wrapper

    return decorator



from .adaptive_async_concurrency_limiter import (
    AdaptiveAsyncConcurrencyLimiter,
    ServiceOverloadError,
)
from .adjustable_semaphore import AdjustableSemaphore
from .raise_on_aiohttp_overload import raise_on_aiohttp_overload
from .raise_on_overload_by_guessing import raise_on_overload
from .with_adaptive_retry import with_adaptive_retry
from .with_async_control import with_async_control

__all__ = [
    "AdaptiveAsyncConcurrencyLimiter",
    "AdjustableSemaphore",
    "raise_on_aiohttp_overload",
    "raise_on_overload",
    "ServiceOverloadError",
    "with_adaptive_retry",
    "with_async_control",
]
